# QA Dashboard
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see
# <http://www.gnu.org/licenses/>.

import json
import logging
import re

from common.management import (
    jenkins_get,
    JenkinsBaseCommand,
)
from common.utils import split_build_number
from suspend.models import (
    SuspendBlocker,
    SuspendImage,
    SuspendResult,
    SuspendMachine,
)
from performance.management import get_image_build_number


def _extract_build_data(
    install_data,
    logs,
    unzip_path,
    online_path,
    log_name,
    build_date,
    kernel,
    image_build_number,
):

    log_path = "{}/{}".format(
        unzip_path,
        log_name,
    )
    log_path_url = "{}/{}".format(
        online_path,
        log_name,
    )
    logging.info("log_path_url: %s", log_path_url)

    install_type = install_data['method']
    dashboard_data = dict(
        release=install_data['release'],
        variant=install_data['variant'],
        arch=install_data['arch'],
        build_date=build_date,
        kernel=kernel,
    )
    if 'jenkins_build' in install_data:
        dashboard_data['jenkins_build'] = install_data['jenkins_build']

    if install_type == 'install':
        dashboard_data['build_number'] = image_build_number
    else:
        dashboard_data['date'] = build_date

        # XXX: if we start having builds that upgrade across
        # releases we will need to update this
        dashboard_data['from_release'] = install_data['release']

    # Assume we need to retrieve the logs if there's no 'unzip_path'
    if 'unzip_path' not in install_data:
        logging.warn("Getting logs from jenkins directly...")
        dashboard_data['output'] = jenkins_get(
            log_path_url,
        )
    else:
        dashboard_data['json'] = log_path

    dashboard_data['log_path'] = log_path
    dashboard_data['log_path_url'] = log_path_url
    dashboard_data['log_name'] = log_name

    return dashboard_data


class Command(JenkinsBaseCommand):

    job_regex = re.compile(
        ur"suspend-blocker-([^-]+)-([^-]+)-([^-]+)"
        "-(install|upgrade)-([^-]+)-(.*)",
    )

    history_days = 30

    def _get_suspend_logs(self, artifacts):
        """ Check for valid suspend blocker results.

        :returns: a list of suspend logs.

        """

        log = None
        suspend_log_regex = re.compile(
            ur'^(clientlogs\/|)(results\/|)(\d+\/|)kern.json',
        )

        for artifact in artifacts:
            logging.info("artifact: %s", artifact)
            path = artifact['relativePath']

            logging.debug("artifact: %s", path)
            if suspend_log_regex.match(path):
                logging.debug("found %s", path)
                return artifact

        return log

    def extract_data(self, name):
        m = self.job_regex.match(name)

        if m:
            self.release = m.group(1)
            self.variant = m.group(2)
            self.arch = m.group(3)
            self.method = m.group(4)
            self.workload = m.group(5)
            self.machine_name = m.group(6)

            logging.info("name: %s", name)
            logging.info("method: %s", self.method)

            self.install_data = dict(
                release=self.release,
                variant=self.variant,
                arch=self.arch,
                method=self.method,
            )

    def add_result(self, dashboard_data):
        """ Add suspend data to the database. """

        logging.info("Adding suspend result.")

        self._get_image(dashboard_data)

        self.jenkins_url = ""
        jenkins_build = dashboard_data.get('jenkins_build')
        if jenkins_build:
            self.jenkins_url = jenkins_build.url

        data = self._parse_log(
            dashboard_data['json'],
        )

        for entry in data['wakelock-stats-from-klog']:

            if not entry:
                continue

            r, new_r = SuspendResult.objects.get_or_create(
                publish=True,
                image=self.image,
                machine=self.machine,
                jenkins_build=jenkins_build,
                defaults=dict(
                    suspend_mean_duration_seconds=(
                        entry['suspend-mean-duration-seconds']),
                    suspend_minimum_duration_seconds=(
                        entry['suspend-minimum-duration-seconds']),
                    suspend_maximum_duration_seconds=(
                        entry['suspend-maximum-duration-seconds']),
                    awake_minimum_duration_seconds=(
                        entry['awake-minimum-duration-seconds']),
                    awake_maximum_duration_seconds=(
                        entry['suspend-maximum-duration-seconds']),
                    suspends_attempted=(
                        entry['suspends-attempted']),
                    suspends_succeeded_percent=(
                        entry['suspends-succeeded-percent']),
                    suspends_total_time_percent=(
                        entry['suspends-total-time-percent']),
                ),
            )

            if not new_r:
                r.suspend_mean_duration_seconds = (
                    entry['suspend-mean-duration-seconds'])
                r.suspend_minimum_duration_seconds = (
                    entry['suspend-minimum-duration-seconds'])
                r.suspend_maximum_duration_seconds = (
                    entry['suspend-maximum-duration-seconds'])
                r.awake_minimum_duration_seconds = (
                    entry['awake-minimum-duration-seconds'])
                r.awake_maximum_duration_seconds = (
                    entry['suspend-maximum-duration-seconds'])
                r.suspends_attempted = (
                    entry['suspends-attempted'])
                r.suspends_succeeded_percent = (
                    entry['suspends-succeeded-percent'])
                r.suspends_total_time_percent = (
                    entry['suspends-total-time-percent'])
                r.save()

            # parse blocking wakelocks
            for wakelock in entry['suspend-blocking-wakelocks']:
                wl, new_wl = SuspendBlocker.objects.get_or_create(
                    publish=True,
                    result=r,
                    name=wakelock['name'],
                    defaults=dict(
                        count=wakelock['count'],
                        percent=wakelock['percent'],
                    ),
                )

                if not new_wl:
                    wl.count = wakelock['count']
                    wl.percent = wakelock['percent']
                    wl.save()

    def process_job(self, job):
        logging.info("job.name: %s", job['name'])
        self.machine, new_machine = SuspendMachine.objects.get_or_create(
            name=self.machine_name,
        )

    def _parse_log(self, filename):
        """ Parse a suspend blocker JSON file. """

        data = {}
        with open(filename, 'r') as fp:
            data = json.load(fp)

        return data

    def _get_image(self, dashboard_data):
        """ Add image. """

        if 'build_number' in dashboard_data:
            image, new_image = SuspendImage.objects.get_or_create(
                release=dashboard_data['release'],
                variant=dashboard_data['variant'],
                arch=dashboard_data['arch'],
                build_number=dashboard_data['build_number'],
                # fake an md5 since we don't have the data
                md5="{}{}{}{}".format(
                    self.arch,
                    self.release,
                    self.variant,
                    dashboard_data['build_number'],
                )
            )

            split_build_number(image)
            self.image = image
            logging.info("adding image: %s", self.image)
        else:
            logging.warn("no build number found. %s",
                         dashboard_data)

    def _process_logs(self, build, build_date, install_data, func):
        """ Process suspend logs calling func() for each log. """

        if not hasattr(func, '__call__'):
            raise Exception("invalid callable: {}".format(func))

        unzip_path = self.get_artifacts(build)
        if unzip_path is None:
            return

        install_data['unzip_path'] = unzip_path

        log = self._get_suspend_logs(build['artifacts'])
        image_build_number = get_image_build_number(build)

        if image_build_number == "UNKNOWN":
            logging.warn("build_number: %s", image_build_number)
            return

        unzip_path = '{}artifact'.format(build['url'])
        online_path = unzip_path

        if 'unzip_path' in install_data:
            unzip_path = "{}/archive".format(install_data['unzip_path'])

        if log:
            log_name = log['relativePath']

            dashboard_data = _extract_build_data(
                install_data,
                log,
                unzip_path,
                online_path,
                log_name,
                build_date,
                "UNKNOWN",
                image_build_number,
            )

            func(dashboard_data)
