# QA Dashboard
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import datetime

from django.core.urlresolvers import reverse
from django.test import TestCase
from django.test.client import Client

from common.models import (
    JenkinsBuild,
    JenkinsJob,
)
from suspend.models import (
    SuspendImage,
    SuspendMachine,
    SuspendResult,
)
from suspend.utils import (
    get_arches,
    get_images,
    get_machines,
)


def _setUp(self):
        self.jenkins_job = JenkinsJob.objects.create(
            name="suspend-blocker-utopic-armhf-touch",
            url="http://example.com",
        )
        self.assert_(self.jenkins_job)
        self.jenkins_build = JenkinsBuild.objects.create(
            job=self.jenkins_job,
            ran_at=datetime.datetime.utcnow(),
            build_number=1,
        )
        self.assert_(self.jenkins_build)

        self.image = SuspendImage.objects.create(
            release='utopic',
            arch='armhf',
            build_number='90:20140620:20140620'
        )
        self.assert_(self.image)

        self.machine = SuspendMachine.objects.create(
            name="mako",
        )
        self.assert_(self.machine)

        self.result = SuspendResult.objects.create(
            image=self.image,
            machine=self.machine,
            jenkins_build=self.jenkins_build,
        )
        self.assert_(self.result)


class SuspendTest(TestCase):

    def setUp(self):
        _setUp(self)

    def test_get_images(self):
        """ Test get_images returns proper results """
        images = get_images(arch=self.image.arch)
        self.assertEqual(1, len(images))
        self.assertEqual(self.image, images[0])

    def test_get_arches(self):
        """ Test get_arches returns proper list of arches. """
        arches = get_arches()
        self.assertEqual(1, len(arches))
        self.assertEqual(self.image.arch, arches[0])

    def test_get_machines(self):
        """ Test get_arches returns proper list of machines. """
        machines = get_machines()
        self.assertEqual(1, len(machines))
        self.assertEqual(self.result.machine, machines[0])


class TestSuspendURLS(TestCase):

    def setUp(self):
        self.c = Client()
        _setUp(self)

    def _test_url(self, url):
        response = self.c.get(url)

        self.assertEqual(response.status_code, 200)

    def test_arch_overview(self):

        url = reverse(
            'suspend_arch_overview',
        )

        self._test_url(url)

    def test_image_overview(self):

        url = reverse(
            'suspend_image_overview',
            args=(
                self.result.image.arch,
            ),
        )

        self._test_url(url)

    def test_result_details(self):

        url = reverse(
            'suspend_result_details',
            args=(
                self.result.image.pk,
            ),
        )

        self._test_url(url)

    def test_suspend_api_machine_overview(self):

        url = reverse(
            'suspend_api_machine_overview',
            args=(
                self.result.machine.id,
            ),
        )

        self._test_url(url)

    def test_suspend_api_images(self):

        url = reverse(
            'suspend_api_images',
            args=(
                self.result.image.arch,
            ),
        )

        self._test_url(url)

    def test_suspend_api_results(self):

        url = reverse(
            'suspend_api_results',
            args=(
                self.result.image.id,
            ),
        )

        self._test_url(url)
